#
# localizer:
# a dumb wrapper around the python 2 codecs that allows rapid encoding/decoding
# of a user-specified locale (like 'iso-8859-1') to/from UTF-8.  it will work
# under python 1, but only for iso-8859-1 (latin-1) as a minimum fallback.
#

# workaround for older (pre-2.0) pythons
import sys, time
if not sys.__dict__.has_key('version_info'):
    # ACK! pre-2.0!
    print '*** Warning: Your python is too old to do any encoding except Latin-1.'
    time.sleep(2)
    OLD_PYTHON = 1
else:
    OLD_PYTHON = 0
    import codecs

def _latin1_to_utf8(str, unused):
    out = ''
    for i in range(len(str)):
        if ord(str[i]) < 128:
            out = out + str[i]
        else:
            out = out + chr(0xC0 + (ord(str[i]) >> 6)) + chr(0x80 + (ord(str[i]) & 0x3F))
    return (out, len(str))

def _utf8_to_latin1(str, unused):
    out = ''
    in_sequence = 0
    buffer = 0
    for i in range(len(str)):
        if ord(str[i]) < 128:
            if in_sequence:
                if buffer > 255:
                    out = out + '?'
                else:
                    out = out + chr(buffer)
                in_sequence = 0
            out = out + str[i]
        else:
            if ord(str[i]) > 0xBF:
                # first byte in a sequence.
                in_sequence = 1
                # this is wrong, but assume the low 5 bits are always the important ones
                # (it'll be right for latin-1 encoded stuff, and i think even for anything
                # in 16-bit unicode)
                buffer = ord(str[i]) & 0x1F;
            else:
                # 6 new bits of the sequence
                buffer = (buffer << 6) + (ord(str[i]) & 0x3F);
    if in_sequence:
        if buffer > 255:
            out = out + '?'
        else:
            out = out + chr(buffer)
    return (out, len(str))

def _nothing(str, unused):
    return (str, len(str))


class Localizer:
    """
    dumb wrapper around the python unicode codecs to make it easy for a say2
    client to convert between the local encoding (default: iso-8859-1) and
    UTF-8 (say2 standard).
    """
    
    def __init__(self, encoding='iso-8859-1'):
        "may raise LookupError"
        self.encoding = encoding
        if OLD_PYTHON:
            self.utf8e, self.utf8d = _nothing, _nothing
            self.locald = _latin1_to_utf8
            self.locale = _utf8_to_latin1
            return
        utf8 = codecs.lookup('utf-8')
        local = codecs.lookup(encoding)
        self.utf8e, self.utf8d, None, None = utf8
        self.locale, self.locald, None, None = local

    def encode(self, str):
        "local encoding -> utf-8"
        unistr, None = self.locald(str, 'replace')
        u8str, None = self.utf8e(unistr, 'replace')
        return u8str
    
    def decode(self, u8str):
        "utf-8 -> local encoding"
        unistr, None = self.utf8d(u8str, 'replace')
        str, None = self.locale(unistr, 'replace')
        return str
    
